<?php
/**
 * WordPress plugin.
 *
 * @author Nicola Mustone <mail@nicolamustone.it>
 * @copyright Copyright (c) 2012 Nicola Mustone
 * @version 1.0
 * @package ycwp-qr-me
 */
 
/**
 * QRCode generator. Uses Google Charts API to generate QR Codes.
 *
 * @author Nicola Mustone <mail@nicolamustone.it>
 * @copyright Copyright (c) 2012 Nicola Mustone
 * @version 1.0
 * @package ycwp-qr-me
 */
class QRCode {
	/**
	 * Base path for QR Code Google request.
     *
     * @access private
     * @var string
     * @since 1.0
	 */
	private $_GOOGLE_CHART_URL;
	
    /**
     * QR Code image width.
     *
     * @access private
     * @var int
     * @since 1.0
     */
	private $_width;
    
    /**
     * QR Code image height.
     *
     * @access private
     * @var int
     * @since 1.0
     */
	private $_height;
	
    /**
     * QR COde image settings.
     *
     * @access private
     * @var array
     * @since 1.0
     */
	private $_settings;
	
    /**
     * Initialize properties.
     */
	public function __construct() {
		$this->_width = 200;
		$this->_height = 200;
		$this->_settings = array(
			'cht' => 'qr',
			'chs' => $this->_width . 'x' . $this->_height,
			'choe' => 'UTF-8',
			'chld' => 'L',
			'chl' => ' '
		);
		
        if( in_array( 'https', stream_get_wrappers() ) ) {
            $this->_GOOGLE_CHART_URL = 'https://chart.googleapis.com/chart?';
        } else {
            $this->_GOOGLE_CHART_URL = 'http://chart.googleapis.com/chart?';
        }
	}
	
    /**
     * Sets QR Code image size.
     *
     * @param int $w
     * @param int $h
     * @return void
     * @since 1.0
     */
	public function setSize($w, $h) {
		$this->setWidth($w);
		$this->setHeight($h);
		
		$size = $this->_width . 'x' . $this->_height;
		$this->_setProperty('chs', $size);
	}
	
    /**
     * Sets QR Code image width.
     *
     * @param int $width
     * @return void
     * @since 1.0
     */
	public function setWidth($w) {
		$this->_width = (int) $w;	
	}
	
    /**
     * Sets QR Code image height.
     *
     * @param int $height
     * @return void
     * @since 1.0
     */
	public function setHeight($h) {
		$this->_height = (int) $h;	
	}
	
    /**
     * Sets QR Code image content charset.
     *
     * @param string $charset
     * @return void
     * @since 1.0
     */
	public function setCharSet($charset) {
		$this->_setProperty('choe', $charset);
	}
	
    /**
     * Sets QR Code image error correction level and margin.
     *
     * @param string $level
     * @param int $margin
     * @return void
     * @since 1.0
     */
	public function setErrorLevel($level, $margin = 0) {
		$this->_setProperty('chld', $level);
		$this->_setProperty('chld', $margin, true);
	}
	
    /**
     * Sets QR Code image content.
     *
     * @param string $content
     * @return void
     * @since 1.0
     */
	public function setContent($content) {
		$this->_setProperty('chl', urlencode($content));
	}
    
    /**
     * Magic method to get properties value.
     *
     * @param string $name Property name
     * @return mixed|bool False on failure
     * @since 1.0
     */
    public function __get( $name ) {
        if( property_exists( $this, $name ) ) {
            return $this->{$name};
        }
        
        return false;
    }
	
    /**
     * Send a GET request to Google for a QR Code.
     *
     * @param array $params Additional params that will be added to the <img> tag.
     * @return string
     */
	public function QR_GET($params = array()) {
		$data = $this->_makeURL();
		$qr = '<img src="' . $data . '"';
			
		if(!empty($params)) {
			foreach($params as $key => $value) {
				$qr .= ' ' . $key . '="' . $value .'"';	
			}
		}
		
		$qr .= ' />';
		
		return $qr;
	}
    
    /**
     * Send a POST request to Google for a QR Code. Required if the content is larger than 2 KB.
     *
     * @param array $params Additional params that will be added to the <img> tag.
     * @return string
     */
    public function QR_POST( $params = array() ) {
        $context = stream_context_create( 
            array( 'http' => 
                array(
                    'method' => 'POST',
                    'header' => 'Content-Type: image/png\r\n',
                    'content' => http_build_query( $this->_settings )
                )
            )
        );
          
        $data = null;
        $fp = fopen( $this->_makeURL(), 'r', false, $context );
        
        while( !feof( $fp ) ) {
            $data .= fread( $fp, 1024 );
        }
          
        $qr = '<img src="data:image/png;base64,' . base64_encode( $data ) . '" ';
        
        if( !empty( $params ) ) {
			foreach( $params as $key => $value ) {
				$qr .= ' ' . $key . '="' . $value .'"';	
			}
		}
        
        $qr .= ' />';
        
        return $qr;
    }
	
    /**
     * Re-initialize the properties
     *
     * @return void
     * @since 1.0
     */
	public function reset() {
		$this->_width = 200;
		$this->_height = 200;
		$this->_settings = array(
			'cht' => 'qr',
			'chs' => $this->_width . 'x' . $this->_height,
			'choe' => 'UTF-8',
			'chld' => 'L',
			'chl' => ''
		);	
	}
	
    /**
     * Set the specified value for a property.
     *
     * @param string $key Property name
     * @param mixed $value
     * @param bool $append Append the value, otherwise it will override
     * @param string $separator
     * @return bool
     * @since 1.0
     */
	protected function _setProperty($key, $value, $append = false, $separator = '|') {
		$params = array_keys($this->_settings);
		
		if($key != 'cht') {
			if(in_array($key, $params)) {
				if(!$append) {
					$this->_settings[$key] = $value;	
				} else {
					$this->_settings[$key] = $this->_settings[$key] . $separator . $value;
				}
				
				return true;
			}
		}
		
		return false;
	}
	
    /**
     * Make the URL for the request.
     *
     * @return string
     * @since 1.0
     */
	protected function _makeURL() {
		$url  = $this->_GOOGLE_CHART_URL;
		$url .= '&amp;cht=' . $this->_settings['cht'];
		$url .= '&amp;chs=' . $this->_settings['chs'];
		$url .= '&amp;chld=' . $this->_settings['chld'];
		$url .= '&amp;chl=' . $this->_settings['chl'];
		$url .= '&amp;choe=' . $this->_settings['choe'];
		
		return $url;
	}
}
?>